
using System;
using System.ComponentModel ;
using System.Runtime.InteropServices;
using System.Windows.Forms ;
using System.Drawing ;
using ExtendedDataGrid.ColumnStyles.Events ;



namespace ExtendedDataGrid.ColumnStyles
{
	/// <summary>
	/// The classic DataGridBoolColumn 
	/// </summary>
	[ComVisibleAttribute(false)]
	public class DataGridBoolColumn : System.Windows.Forms.DataGridBoolColumn, ExtendedDataGrid.ICommonColumnStyle	
	{
		private ResizeType resizeType=ResizeType.Fixed ;
		private int colIndex=-1;
		private double widthPercent=0;

		#region <<  Constructors  >>

		public DataGridBoolColumn():this(null){}

		public DataGridBoolColumn(PropertyDescriptor prop):base(prop)
		{
		}


		#endregion 

		[Description("Specify how the column will resize when the grid will resize"),
		DefaultValue(ResizeType.Fixed ),Category("Behaviour")]
		ResizeType ICommonColumnStyle.ResizeType 
		{
			get{return this.resizeType ;}
			set{this.resizeType=value;}
		}

		double ICommonColumnStyle.WidthPercent 
		{
			get{return this.widthPercent;}
			set{this.widthPercent=value;}
		}


		[ReadOnly(true),Browsable(false),Description("The index of this column in the current table style ")]
		public int ColumnIndex
		{
			get
			{
				if(this.colIndex==-1)
					if(this.DataGridTableStyle!=null && this.DataGridTableStyle.GridColumnStyles.Contains(this))
						this.colIndex=this.DataGridTableStyle.GridColumnStyles.IndexOf(this);

				return this.colIndex;
			}
		}

		protected  override bool Commit(CurrencyManager dataSource, int rowNum)
		{
			base.Commit(dataSource,rowNum);
			return true;

		}


		protected override void Abort(int rowNum)
		{
			base.Abort(rowNum);
			base.Invalidate();
		}

		protected  override void ReleaseHostedControl()
		{
			this.colIndex=-1;
			base.ReleaseHostedControl ();
			this.Invalidate();
		}

		public void Refresh()
		{
		this.Invalidate();
		}
		

		protected override void SetDataGridInColumn(System.Windows.Forms.DataGrid value)
		{
			base.SetDataGridInColumn (value);
		}
		
	}




}
