using System;
using System.Windows.Forms ;
using System.Drawing;
using System.ComponentModel ;
using System.Data ;
using System.Reflection ;
using System.Globalization;
using ExtendedDataGrid.ColumnStyles.Events;
using ExtendedDataGrid.Controls ;
using System.Diagnostics ;

namespace ExtendedDataGrid.ColumnStyles
{
	[DefaultEvent("SetCellStyle")]
	public class DataGridDTPickerColumn: DataGridColumnStyleBase
	{
		#region <<Fields >> 

		private DTPicker  dtp=new DTPicker();
		private TextBox txtBox=new TextBox();
		private ImageList imgList=null;
		private int defaultImageIndex=-1;
		private bool isInEditing =false;
		private int offset=0; // the distance between the immage and text
		private ResizeType resizeType=ResizeType.Proportional ; 
		private event DTPickerColumnSetCellStyleEventHandler cellStyle=null;
		private object oldValue=DBNull.Value;
		private object currentRow=null;
		private int xMargin=2;
		private int yMargin=1;
		private bool editingBegun=false;

		private bool defaultAllowKeyboardInput=true;
		private bool defaultShowComboButton=true ;

		//private BindingType bindTo=BindingType.SelectedValue ;
		
		#endregion Private Fields 

		#region >> Constructors <<

		public DataGridDTPickerColumn():this(null){}


		public DataGridDTPickerColumn(PropertyDescriptor prop):base(prop)
		{
			this.ReadOnly=false;
			this.dtp.Hide();
			base.Alignment=HorizontalAlignment.Center ;
			this.dtp.TextAlign=HorizontalAlignment.Center;
			this.txtBox.Hide();
			this.dtp.ValueModifiedWithPicker+=new EventHandler(dtp_ValueModifiedWithPicker);
			this.dtp.DateEditControl.KeyPress+=new KeyPressEventHandler(DateEditControl_KeyPress);
		}




		#endregion Constructors 

		#region <<  Properties  >>

		[DefaultValue(null),RefreshProperties(RefreshProperties.Repaint)]
		public ImageList ImageList
		{
			get
			{
				return this.imgList ;
			}
			set 
			{
				this.imgList=value ;
				this.dtp.ImageList=value;
				if (value==null)
				{
					this.offset=0;
					this.defaultImageIndex=-1;
				}
				else
					this.BorderStyle=this.BorderStyle;

			}
		}
		
		[DefaultValue("(null)")]
		public override string NullText
		{
			get
			{
				return this.dtp.NullDisplayText ;
			}
			set
			{
				base.NullText = value;
				this.dtp.NullDisplayText =value;
			}
		}

//		/// <summary>
//		/// The default value for th readonly property of this column.
//		/// Use the CurrentReadOnly property of the parameter in the event SetCellStyle
//		/// </summary>
//		[DefaultValue(false),Category("Behaviour")]
//		public bool DefaultReadOnly 
//		{
//			get{return this.defaultReadOnly;}
//			set{
//				this.defaultReadOnly=value;
//				this.ReadOnly=value;
//				if(this.dtp.Visible)
//					this.dtp.ReadOnly=value;
//			}
//		}


		[Browsable(false),EditorBrowsable(EditorBrowsableState.Never )]
		public override bool ReadOnly
		{
			get
			{
				return base.ReadOnly;
			}
			set
			{
				base.ReadOnly = value;
			}
		}

		/// <summary>
		/// Shows/Hides the combo button of the DateEdit Control
		/// </summary>
		[DefaultValue(true),Browsable(true),Category("Behaviour"),Description("Shows/Hides the combo button of the DateEditControl")]
		public bool DefaultShowComboButton
		{
			get{return this.defaultShowComboButton ;}
			set{this.defaultShowComboButton=value;this.dtp.ShowComboButton =value;}
		}

		/// <summary>
		/// Gets/Sets whether the modifications with the keybord are accepted 
		/// </summary>
		/// <returns></returns>
		[DefaultValue(true),Browsable(true),Category("Behaviour")]
		public bool DefaultAllowKeyboardInput
		{
			get{return this.defaultAllowKeyboardInput ;}
			set{
				this.defaultAllowKeyboardInput=value;
				this.dtp.AllowKeyboardInput=value;
				this.Invalidate();
			}
		}
		
		/// <summary>
		/// The default index of the image to show in this column 
		/// </summary>
		[Description("The default index of the image to show in this column"),
		DefaultValue(-1),Category("Layout"),
		TypeConverter(typeof(ImageIndexConverter)),System.ComponentModel.EditorAttribute("System.Windows.Forms.Design.ImageIndexEditor",typeof(System.Drawing.Design.UITypeEditor))]
		public int DefaultImageIndex 
		{
			get
			{
				return this.defaultImageIndex ;
			}
			set
			{
				this.defaultImageIndex=value;
				this.dtp.ImageIndex=value;
			}
		}


		/// <summary>
		/// Gets/Sets the way this column will resize when the host grid resizes
		/// </summary>
		[Description("Gets/Sets the way this column will resize when the host grid resizes"),
		DefaultValue(ResizeType.Proportional ),Category("Behaviour")]
		public override ResizeType ResizeType
		{
			get{return this.resizeType ;}
			set{this.resizeType=value;}
		}
	

		/// <summary>
		/// This event raises every time the cells in this column should paint 
		/// </summary>
		[Description("This event raises every time the cells in this column should paint ")]
		public event DTPickerColumnSetCellStyleEventHandler SetCellStyle
		{
			add 
			{
				this.cellStyle+=value ;
			}
			remove
			{
				this.cellStyle-=value;
			}
		}


		[Browsable(false),ReadOnly(true)]
		public DTPicker DTPicker 
		{
			get{return this.dtp;}
		}


		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public DateTimeFormatInfo DateTimeFormatInfo 
		{
			get{return this.dtp.DateTimeFormatInfo ;}
			set{this.dtp.DateTimeFormatInfo=value;}
		}

		public void ResetDateTimeFormatInfo()
		{this.dtp.DateTimeFormatInfo=Application.CurrentCulture.DateTimeFormat; }


		/// <summary>
		/// The character used to mark inputs in the EditMask
		/// </summary>
		[DefaultValue('_'),Description("The character used to mark inputs in the EditMask")]
		public Char InputChar
		{
			get{return this.dtp.InputChar ;}
			set{this.dtp.InputChar=value;}
		}


		
		/// <summary>
		/// The format used for displaying the Value property when the control is not in edit mode
		/// </summary>
		[Description("The format used for displaying the Value property when the control is not in edit mode")]
		public string DisplayFormat
		{
			get{return this.dtp.DisplayFormat;}
			set{this.dtp.DisplayFormat=value;}
		}

		public bool ShouldSerializeDisplayFormat()
		{
			return this.dtp.ShouldSerializeDisplayFormat();
		}
		
		/// <summary>
		/// The Value property of the DTPicker control inside this column
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),Browsable(false)]
		public object Value
		{
			get{return this.dtp.Value ;}
			set{this.dtp.Value=value;}
		}


		/// <summary>
		/// The mask for input data. It is determined by the EditFormat property
		/// </summary>
		[ReadOnly(true),Description("The mask for input data. It is determined by the EditFormat property")]
		public string EditMask
		{
			get{return this.dtp.EditMask; }
		}

		

		/// <summary>
		/// The format used when the control is in edit mode. It determines the EditMask .
		/// </summary>
		[Browsable(true), Description("The format used when the control is in edit mode. It determines the EditMask .")]
		public string EditFormat
		{
			get{return this.dtp.EditFormat;}

			set{this.dtp.EditFormat=value;}
		}


		public bool ShouldSerializeEditFormat()
		{
			return this.dtp.ShouldSerializeEditFormat();
		}


		/// <summary>
		/// Indicate whether th text value in the textbox of this control can be converted to a valid DateTime value
		/// </summary>
		[ReadOnly(true),Browsable(false)]
		public bool IsDateValid
		{
			get{return this.dtp.IsDateValid  ;}
		}

		/// <summary>
		/// You should check the IsDateValid before calling this method.
		/// If the text(the user has input) is cannot be converted to a valid DateTime value,
		/// a FormatException will be raised   
		/// </summary>
		/// <returns>Return a DateTime representation of the edited text</returns>
		public DateTime GetDateTimeValue()
		{
		return this.dtp.DateTimeValue() ;
		}

		/// <summary>
		/// Gets the Text value of the TextBox control as is, without formatting it  
		/// </summary>
		/// <returns></returns>
		public string Text()
		{
		return this.dtp.Text;
		}

		/// <summary>
		/// Returns the formatted Text value of this control
		/// Returns the FormattedText property of the DTPicker control inside this column
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),Description("Returns the formatted Text value of this control"),
		ReadOnly(true),Browsable(true),]
		public string FormattedText 
		{
			get{return this.dtp.FormattedText   ;}
		}
		
		public Font CalendarFont
		{
			get{return this.dtp.CalendarFont;}
			set{this.dtp.CalendarFont=value;}
		}

		public bool ShouldSerializeCalendarFont()
		{
			return this.dtp.CalendarFont!=new Font(@"Microsoft Sans Serif", 8.25f);
		}

		public void ResetCalendarFont()
		{
			this.dtp.CalendarFont=new Font(@"Microsoft Sans Serif", 8.25f);
		}
		
		public Color CalendarForeColor 
		{
			get{return this.dtp.CalendarForeColor ;}
			set{this.dtp.CalendarForeColor=value;}
		}
		public bool ShouldSerializeCalendarForeColor()
		{
			return this.dtp.CalendarForeColor!=System.Drawing.SystemColors.ControlText;
		}
		public void ResetCalendarForeColor()
		{
			this.dtp.CalendarForeColor=System.Drawing.SystemColors.ControlText;
		}


		public Color CalendarMonthBackground 
		{
			get{return this.dtp.CalendarMonthBackground  ;}
			set{this.dtp.CalendarMonthBackground=value;}
		}
		public bool ShouldSerializeCalendarMonthBackground()
		{
			return this.dtp.CalendarMonthBackground!=System.Drawing.SystemColors.Window;
		}

		public void ResetCalendarMonthBackground()
		{
			this.dtp.CalendarMonthBackground=System.Drawing.SystemColors.Window;
		}


		public Color CalendarTitleBackColor 
		{
			get{return this.dtp.CalendarTitleBackColor  ;}
			set{this.dtp.CalendarTitleBackColor=value;}
		}

		public bool ShouldSerializeCalendarTitleBackColor()
		{
			return this.dtp.CalendarTitleBackColor!=System.Drawing.SystemColors.ActiveCaption;
		}

		public void ResetCalendarTitleBackColor()
		{
			this.dtp.CalendarTitleBackColor=System.Drawing.SystemColors.ActiveCaption;
		}


		public Color CalendarTitleForeColor 
		{
			get{return this.dtp.CalendarTitleForeColor  ;}
			set{this.dtp.CalendarTitleForeColor=value;}
		}
		public bool ShouldSerializeCalendarTitleForeColor()
		{
			return this.dtp.CalendarTitleForeColor!=System.Drawing.SystemColors.ActiveCaptionText;
		}

		public void ResetCalendarTitleForeColor()
		{
			this.dtp.CalendarTitleForeColor=System.Drawing.SystemColors.ActiveCaptionText;
		}


		public Color CalendarTrailingForeColor 
		{
			get{return this.dtp.CalendarTrailingForeColor  ;}

			set{this.dtp.CalendarTrailingForeColor=value;}
		}
		public bool ShouldSerializeCalendarTrailingForeColor()
		{
			return this.dtp.CalendarTrailingForeColor!=System.Drawing.SystemColors.GrayText;
		}

		public void ResetCalendarTrailingForeColor()
		{
			this.dtp.CalendarTrailingForeColor=System.Drawing.SystemColors.GrayText;
		}


		[DefaultValue(BorderStyle.Fixed3D)]
		public BorderStyle BorderStyle
		{
			get{return this.dtp.BorderStyle;}

			set
			{
				this.dtp.BorderStyle=value;
				if (value==BorderStyle.Fixed3D) 
					this.offset=2;
				else if(value==BorderStyle.FixedSingle)
					this.offset=1;
				else
					this.offset=0;
			}
		}


		protected override void SetDataGridInColumn(System.Windows.Forms.DataGrid value)
		{
			base.SetDataGridInColumn (value);
			this.dtp.Width=0;
			if (this.dtp.Parent!=null)
				this.dtp.Parent.Controls.Remove(this.dtp);
			if(this.txtBox.Parent!=null)
				this.txtBox.Parent.Controls.Remove(this.txtBox);

			if(value!=null)
			{
				value.Controls.Add(this.txtBox);
				value.Controls.Add(this.dtp);
			}
		}


		#endregion Properties 

		#region <<  Overridings  >>


		protected override void Abort(int rowNum)
		{
			
			this.isInEditing=false;
			this.currentRow=null;
			this.dtp.Hide();
			this.Invalidate();
		}
 

		protected override bool Commit(CurrencyManager dataSource, int rowNum)
		{
			object val=this.dtp.Value ;
			bool success=false ;
			this.dtp.Hide();
			if(this.isInEditing && !val.Equals(this.oldValue))
			{
				try
				{

					
					if(rowNum!=dataSource.Position )
					{					  
							success=Util.SetValueInRow(dataSource.Current,this.MappingName,val);
					}
					else
					{
						if( !this.GetColumnValueAtRow(dataSource,rowNum ).Equals(val))
						{
							this.SetColumnValueAtRow(dataSource,rowNum,val);
						}
					}
					success=true;
				}
				catch(Exception ex)
				{
					success=false;
					throw new Exception("DataGridDTPickerColumn.Commit\n\n"+ex.Message);
				}
				this.currentRow=null;
			}
			this.oldValue=DBNull.Value ;
			this.isInEditing=false ;
			this.Invalidate();


		return true;

		}

		protected override void Dispose(bool disposing)
		{
			base.Dispose (disposing);
		}

		[DefaultValue(System.Windows.Forms.HorizontalAlignment.Center )]
		public override HorizontalAlignment Alignment
		{
			get
			{
				return base.Alignment;
			}
			set
			{
				base.Alignment = value;
				this.dtp.TextAlign=value;
			}
		}

//		protected override Size GetPreferredSize(Graphics g, object value)
//		{
//			return this.dtp.Size ;
//		}


		internal override void ReleaseFocus()
		{
			base.ReleaseFocus();
			this.ConcedeFocus();
		}


//		protected override int GetMinimumHeight()
//		{
//			return  this.dtp.Height;
//		}
//
//
//		protected override int GetPreferredHeight(Graphics g, object value)
//		{
//			return  this.dtp.Height;
//		}
 
		protected override void ReleaseHostedControl()
		{
			base.ReleaseHostedControl ();

			if (this.dtp.Parent!=null)
				this.dtp.Parent.Controls.Remove(this.dtp );
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly)
		{
			this.Edit(source,rowNum,bounds,readOnly,"",true);
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly, string instantText)
		{
			this.Edit (source, rowNum, bounds, readOnly, instantText,true);
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly, string instantText, bool cellIsVisible)
		{
			base.Edit(source,rowNum,bounds,readOnly,instantText,cellIsVisible);

			this.currentRow=source.Current ;

			if(cellIsVisible )
			{
				
				//DebugOut("Edit rowNum="+rowNum.ToString()+"     this.editingBegun="+this.editingBegun.ToString() );
				
				System.Windows.Forms.DataGrid hostGrid=this.DataGridTableStyle.DataGrid ;

				DTPickerColumnSetCellStyleEventArgs e;
				if (rowNum % 2 ==0)
					e=new DTPickerColumnSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),hostGrid.BackColor,hostGrid.ForeColor,this.defaultImageIndex ,true ,this.defaultShowComboButton,this.defaultAllowKeyboardInput  );
				else
					e=new DTPickerColumnSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),hostGrid.AlternatingBackColor ,hostGrid.ForeColor,this.defaultImageIndex,true,this.defaultShowComboButton,this.defaultAllowKeyboardInput );
				this.OnSetCellStyle(e);
			
				object currentValue=this.GetColumnValueAtRow(source,rowNum);

				this.dtp.Bounds=bounds;
				this.dtp.ImageIndex=e.CurrentImageIndex ;

				this.dtp.BackColor= e.CurrentBackColor ;
				this.dtp.ForeColor=e.CurrentForeColor ;
				this.dtp.Show();
				this.dtp.Select();
				this.dtp.Focus();
				this.dtp.Value= currentValue;
				this.oldValue=this.dtp.Value ;

				if(!e.CurrentAllowKeyboardInput )
					this.dtp.SelectTextAll();
				this.dtp.ShowComboButton =e.CurrentShowComboButton ;

				this.isInEditing=true;
			}
		}

		
		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum)
		{
			this.Paint(g,bounds,source,rowNum,null,null,false);
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum, bool alignToRight)
		{
			this.Paint(g,bounds,source,rowNum,null,null,alignToRight);
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum, Brush backBrush, Brush foreBrush, bool alignToRight)
		{
			
			Color backColor , foreColor;
			this.GetPaintColors(out backColor,out foreColor,rowNum,source.Position );

			//genero l'evento SetCellStyle
			DTPickerColumnSetCellStyleEventArgs e;

			object objValue=this.GetColumnValueAtRow(source,rowNum);

			e=new DTPickerColumnSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),backColor,foreColor, this.defaultImageIndex,false, this.defaultShowComboButton,this.defaultAllowKeyboardInput);
			
			this.OnSetCellStyle(e);


			// prepare things for drawing  

			Rectangle imgRect=Rectangle.Empty;
			Rectangle textRect=Rectangle.Empty;

			this.BeginUpdate();

			// clear the background of the cell ;

			g.FillRectangle(new SolidBrush(e.CurrentBackColor),bounds);

			bounds.Inflate(-2,0);

			//

			if ( this.imgList!=null)
				imgRect=this.GetImageRectangle(bounds);

			textRect=GetComboBoxRectangle(bounds,imgRect,this.dtp.Font,false );



			if(!textRect.IsEmpty)
			{
				string txt="Invalid data";
				if(objValue.Equals(DBNull.Value))
				{
					txt=this.NullText;
				}
				else
				{
					DateTime dt;
					try
					{
						dt=(DateTime)objValue;
						txt=dt.ToString(this.dtp.DisplayFormat,this.dtp.DateTimeFormatInfo );
					}
					catch(Exception ex)
					{
						throw new Exception(ex.Message );
					}
				}
				Brush brush=new SolidBrush(e.CurrentForeColor);
				StringFormat fmt=new StringFormat();
				fmt.Alignment=(this.Alignment==HorizontalAlignment.Center?StringAlignment.Center :(this.Alignment==HorizontalAlignment.Left ?StringAlignment.Near:StringAlignment.Far)) ;
				g.DrawString(txt ,this.dtp.Font ,brush, textRect,fmt);

				brush.Dispose();
			}


			//draw the image
			if(!imgRect.IsEmpty && e.CurrentImageIndex>-1 && e.CurrentImageIndex<this.imgList.Images.Count )
			{
				Rectangle srcImgRect=new Rectangle(0,0,(this.imgList.ImageSize.Width>bounds.Width?bounds.Width:this.imgList.ImageSize.Width),(this.imgList.ImageSize.Height>bounds.Height?bounds.Height:this.imgList.ImageSize.Height )  );
				try
				{
					g.DrawImage(this.imgList.Images[e.CurrentImageIndex],imgRect,srcImgRect,GraphicsUnit.Pixel );
				}
				catch (ArgumentOutOfRangeException ex)
				{
					throw new ArgumentOutOfRangeException("ImageIndex",e.CurrentImageIndex ,"DataGridDTPickerColumn.Paint Error:\nThe specified image index does not exist in the ImageList\n"+ex.Message );
				}
				catch(Exception ex) 
				{
					throw new Exception(ex.Message );
				}
			}
			this.EndUpdate();
		}


		protected override void ConcedeFocus()
		{
			this.editingBegun=false;
			this.dtp.Hide();
			this.isInEditing=false;
			base.ConcedeFocus ();
		}

		

		#endregion Overridings

		#region << Private methods >>


		private void GetPaintColors(out Color backColor, out Color foreColor ,  int rowNum , int sourcePosition)
		{
			DataGridTableStyle tableStyle=this.DataGridTableStyle ;
			if (rowNum==sourcePosition )
			{
				//foreColor=hostGrid.SelectionForeColor  ;
				if (this.isInEditing) 
				{
					backColor=tableStyle.BackColor  ;
					foreColor=tableStyle.ForeColor ;
				}
				else
				{
					backColor=tableStyle.SelectionBackColor ;
					foreColor=tableStyle.SelectionForeColor ;
				}
			}
			else
			{
				foreColor=tableStyle.ForeColor;
				if (rowNum % 2 ==0)
				{
					backColor=tableStyle.BackColor;
				}
				else 
				{
					backColor=tableStyle.AlternatingBackColor;
				}
			}

			//Debug.Assert(foreColor!=Color.White );
		}



		private Rectangle GetImageRectangle(Rectangle cellBounds)
		{
			if (this.imgList==null)
				return Rectangle.Empty;
			Rectangle rctResult=Rectangle.Empty;
			if (this.imgList.ImageSize.Height<cellBounds.Height )
			{
				int top =0;
				top=(cellBounds.Height-this.imgList.ImageSize.Height )/2;
				rctResult=new Rectangle(cellBounds.X+this.offset,cellBounds.Y+top,this.imgList.ImageSize.Width,this.imgList.ImageSize.Height  );
			}
			else
				rctResult=new Rectangle(cellBounds.X+this.offset,cellBounds.Y,this.imgList.ImageSize.Width,cellBounds.Height);
			return rctResult;
		}


		private Rectangle GetComboBoxRectangle(Rectangle cellBounds,Rectangle rctImage, Font font,bool calledFromEdit )
		{
			if (calledFromEdit)
			{
				cellBounds.Offset(this.xMargin, (2 * this.yMargin));
				cellBounds.Width -= 2*this.xMargin;
				cellBounds.Height-= 2 * this.yMargin; 
			}
			else
			{
				cellBounds.Offset(0, 2 * this.yMargin);
				cellBounds.Height -= 4 * this.yMargin;
				cellBounds.Width-=this.dtp.ComboButtonWidth ;
			}


			int imgSpace=0;
			if(!rctImage.IsEmpty)
				imgSpace=rctImage.Width  +2*this.offset;

			Rectangle rctTxtBox=Rectangle.Empty;
			int top=0;
			if(imgSpace <=cellBounds.Width  )
			{
				top=(cellBounds.Height>font.Height?(cellBounds.Height-font.Height)/2:0);

				rctTxtBox=new Rectangle(cellBounds.X+imgSpace ,
					cellBounds.Y+top,
					cellBounds.Width-imgSpace,
					font.Height );
			}

			return rctTxtBox;
		}

		

		#endregion Private methods 

		#region >>  ProtectedMethods  <<

		protected virtual void OnSetCellStyle(DTPickerColumnSetCellStyleEventArgs e)
		{
			if(this.cellStyle!=null)
			{
					this.cellStyle(this,e);
			}
		}

		#endregion ProtectedMethods

		private void dtp_ValueModifiedWithPicker(object sender, EventArgs e)
		{
			if(this.isInEditing && this.ReadOnly==false && this.editingBegun==false )
			{
				this.editingBegun=true;
				this.ColumnStartedEditing(this.dtp);
			}
		}


		private void DateEditControl_KeyPress(object sender, KeyPressEventArgs e)
		{
			if(this.ReadOnly==false && this.editingBegun==false  && Char.IsDigit(e.KeyChar )  )
			{
				this.editingBegun=true;
				this.ColumnStartedEditing(this.dtp);

			}
		}

		private void DebugOut(string text)
		{
			Debug.WriteLine (text+"      Class:DataGridDTPickerColumn");
		}

	}//end class DataGridFormatDTPickerColumn



namespace Events
{
	public delegate void DTPickerColumnSetCellStyleEventHandler(object sender,DTPickerColumnSetCellStyleEventArgs e);

	/// <summary>
	/// Class SetCellStyleEventArgs derived from EventArgs 
	/// The type of the second parameter  in the event SetCellStyle 
	/// </summary>
	public class DTPickerColumnSetCellStyleEventArgs:EventArgs
	{
		#region Private Fields 
		private bool isInEditing=false;
		private int currentRowNumber=-1;
		private Color currentBackColor;
		private Color currentForeColor;
		private int currentImageIndex;
		//private bool currentReadOnly ;
		private bool currentAllowKeyboardInput ;
		private bool currentShowComboButton;
		#endregion Private Fields 

		#region Constructors
		public DTPickerColumnSetCellStyleEventArgs(int curRowIndex,Color curBackColor, Color curForeColor,int curImageIndex, bool inEditing,bool curShowComboButton, bool curAllowKeyboardInput  )
		{
			this.currentRowNumber=curRowIndex ;
			this.isInEditing=inEditing ;
			this.currentBackColor=curBackColor;
			this.currentForeColor=curForeColor;
			this.currentImageIndex=curImageIndex;
			this.currentShowComboButton=curShowComboButton;
			this.currentAllowKeyboardInput=curAllowKeyboardInput;
		}
		#endregion constructors 

		#region Properties

		
//		/// <summary>
//		/// Gets/Sets the ReadOnly state of the cell for which this event has been raised 
//		/// </summary>
//		public bool CurrentReadOnly
//		{
//			get{return this.currentReadOnly;}
//			set{this.currentReadOnly=value;}
//		}



		/// <summary>
		/// ReadOnly .
		/// Indicates whether the cell(for which this event has been raised) is being edited
		/// </summary>
		[ReadOnly(true)]
		public bool IsInEditing
		{
			get
			{
				return this.isInEditing ;
			}
		}

		/// <summary>
		/// Gets/Sets the back color of the  cell for which this event has been raised 
		/// </summary>
		public Color CurrentBackColor
		{
			get
			{
				return this.currentBackColor;
			}
			set
			{
				this.currentBackColor=value;
			}
		}

		/// <summary>
		/// Gets/Sets the ForeColor of the  cell for which this event has been raised 
		/// </summary>
		public Color CurrentForeColor
		{
			get
			{
				return this.currentForeColor;
			}
			set
			{
				this.currentForeColor=value;
			}
		}

		/// <summary>
		/// Gets/Sets the index of the image to show in the  cell  for which this event has been raised 
		/// </summary>
		public int CurrentImageIndex
		{
			get
			{
				return this.currentImageIndex ;
			}
			set
			{
				this.currentImageIndex=value;
			}
		}


		/// <summary>
		/// ReadOnly
		/// The index of the row for which this event has been raised
		/// </summary>
		[ReadOnly(true)]
		public int CurrentRowNumber
		{
			get
			{
				return this.currentRowNumber ;
			}
		}

		/// <summary>
		/// Gets/Sets a boolean value indicating whether the control accepts keyboard inputs 
		/// </summary>
		/// <returns></returns>
		public bool CurrentAllowKeyboardInput
		{
			get{return this.currentAllowKeyboardInput;}
			set{this.currentAllowKeyboardInput=value;}
		}

		public bool CurrentShowComboButton
		{
			get
			{
			return this.currentShowComboButton;
			}
			set
			{
			this.currentShowComboButton=value;
			}
		}

		#endregion Properties 
	}

} // end namespace Events

}