using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using System.Diagnostics;


namespace ExtendedDataGrid
{
	/// <summary>
	/// This class is the Edit control for the class DataGridFormatTextBoxColumn.
	/// </summary>
	[ToolboxBitmap(typeof(FormatTextBox)),DefaultProperty("Text"),DefaultEvent("EllipsisClick")]
	public class DataGridFormatTextBox : System.Windows.Forms.UserControl
	{
		
		#region <<  FIELDS >>

		private DataGridTextBox edit;
		private Rectangle rectImage=Rectangle.Empty;
		private Rectangle rectEllipsis=Rectangle.Empty ;
		private ImageList imgList =null;
		private int imageIndex =-1;
		private bool showEllipsis=true;
		private const int ellipsisWidth=16;
		private const int sp=2 ; // space(in pixels) between image<->Textbox and also between Textbox<->ellipsis button  
		private int border = 2; 
		private bool mouseIsEntered =false ;
		private bool mouseIsDown=false ;
		private event EventHandler ellipsis_Click=null;
		private EllipsisBackgroundStyle ellipsisBackStyle=EllipsisBackgroundStyle.GradientCentered ;
		private BorderStyle borderStyle=BorderStyle.Fixed3D ;

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		#endregion

		#region     <<  CONSTRUCTORS  >>

		public DataGridFormatTextBox()
		{
			InitializeComponent();
			base.BackColor=Color.White ;
			base.Height=24;
			this.edit.MultilineChanged+=new EventHandler(edit_MultilineChanged);
			this.edit.FontChanged+=new EventHandler(edit_FontChanged);
			this.edit.BorderStyleChanged+=new EventHandler(edit_BorderStyleChanged);
			this.EnabledChanged+=new EventHandler(FormatTextBox_EnabledChanged);
			this.edit.Font= new Font("Arial",10);
			this.edit.TextChanged+=new EventHandler(edit_TextChanged);
			
			RedimControls();
		}

		#endregion 


		#region    <<  PROPERTIES >>


//
		public event EventHandler EllipsisClick
		{
			add{this.ellipsis_Click+=value;}
			remove{this.ellipsis_Click+=value;}
		}

		[ReadOnly(true),Browsable(false)]
		public int EllipsisWidth
		{
			get{return DataGridFormatTextBox.ellipsisWidth ;}
		}
		[Localizable(true),Bindable(true),Browsable(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Content )]
		public override string Text
		{
			get{return this.edit.Text;}
			set{
				base.Text=value;
				this.edit.Text=value;
				base.OnTextChanged(new EventArgs());
			}
		}
		
		public new Font Font
		{
			get{return this.edit.Font ;}
			set{
				if(value!=null)
					this.edit.Font=value;
				
				
			}
		}
		protected bool ShoulSerializeFont()
		{
		return this.edit.Font!=new Font("Arial",10);
		}

		protected new  void ResetFont()
		{
		this.edit.Font= new Font("Arial",10);
		}

		[DefaultValue(0)]
		public int MaxLength
		{
			get{return this.edit.MaxLength;}
			set{this.edit.MaxLength =value;}
		}


		[DefaultValue(EllipsisBackgroundStyle.GradientCentered)]
		public EllipsisBackgroundStyle EllipsisBackStyle
		{
			get{return this.ellipsisBackStyle ;}
			set{this.ellipsisBackStyle=value;this.Invalidate();}
		}

		[DefaultValue(BorderStyle.Fixed3D )]
		public BorderStyle BorderStyle
		{
			get{return this.borderStyle  ;}
			set{this.borderStyle=value;this.Invalidate();}
		}

		[DefaultValue(null)]
		public ImageList ImageList 
		{
			get{return this.imgList;}
			set{
				this.imgList=value;
				if(value==null)
					this.imageIndex=-1;
				this.Invalidate();
			}
		}


		[DefaultValue(HorizontalAlignment.Left )]
		public HorizontalAlignment TextAlign 
		{
			get{return this.edit.TextAlign ;}
			set{this.edit.TextAlign=value;}
		}


		[DefaultValue(-1),TypeConverter(typeof(ImageIndexConverter)),System.ComponentModel.EditorAttribute("System.Windows.Forms.Design.ImageIndexEditor",typeof(System.Drawing.Design.UITypeEditor))]
		public int ImageIndex
		{
			get{return this.imageIndex;}
			set{this.imageIndex=value;this.RedimControls();this.Invalidate();}
		}

		
		[DefaultValue(true)]
		public bool ShowEllipsis
		{
			get{return this.showEllipsis ;}
			set{this.showEllipsis=value;this.RedimControls();this.Invalidate();}
		}
		

		[ReadOnly(true)]
		public new Rectangle ClientRectangle
		{
			get{
			Rectangle r = base.ClientRectangle ;
			r.Inflate(-this.border,-this.border );
			return r;
			}
		}

		public bool Multiline
		{
			get{return this.edit.Multiline ;}
			set{this.edit.Multiline=value;}
		}

		[ReadOnly(true),Browsable(false)]
		public DataGridTextBox TextBox 
		{
			get{return this.edit;}
		}

		#endregion 

		#region <<  OVERRIDES  >>


		public override Color BackColor
		{
			get
			{
				return base.BackColor;
			}
			set
			{
				base.BackColor = value;
				this.edit.BackColor=value;
				this.Invalidate();
			}
		}

		
		
		public override Color ForeColor
		{
			get
			{
				return base.ForeColor;
			}
			set
			{
				base.ForeColor = value;
				this.edit.ForeColor=value;
			}
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			if(this.showEllipsis  && e.X>this.rectEllipsis.X)
			{
				this.mouseIsDown=true;
				this.Invalidate();
			}
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			base.OnMouseUp (e);
			if(this.mouseIsDown)
			{
			this.mouseIsDown=false;
			this.Invalidate();
			this.OnEllipsisClick(new EventArgs());
			this.Invalidate();
			}
		}

		protected override void OnMouseMove(MouseEventArgs e)
		{
			base.OnMouseMove (e);
			bool oldVal=this.mouseIsEntered;
			if(this.showEllipsis==false)
				return ;

			if(e.X>this.rectEllipsis.X)
				{
					this.mouseIsEntered=true;
					this.Cursor=Cursors.Hand  ;
				}
				else
				{
					this.mouseIsEntered=false;
					this.Cursor=Cursors.Default ;
				}

			if(oldVal!=this.mouseIsEntered )
				this.Invalidate();
		}

		protected override void OnMouseLeave(EventArgs e)
		{
			base.OnMouseLeave (e);
			if(this.mouseIsEntered)
			{
				this.mouseIsEntered=false;
				this.Cursor=Cursors.Default ;
				this.Invalidate();
			}
		}
		protected override void OnLeave(EventArgs e)
		{
			base.OnLeave (e);
			this.mouseIsDown=false;
		}


		[DefaultValue(false)]
		public bool ReadOnly 
		{
			get{return this.edit.ReadOnly;}
			set{this.edit.ReadOnly =value;}
		}


		

		protected override void OnResize(EventArgs e)
		{
			base.OnResize (e);
			this.RedimControls();
		}

		protected override void OnTextChanged(EventArgs e)
		{
			base.OnTextChanged (e);
		}

		private void GetColors(out Color backColor,out Color foreColor )
		{
			if(this.Enabled )
			{
				backColor=this.BackColor ;
				foreColor=this.ForeColor;
			}
			else
			{
				if(this.BackColor==SystemColors.Window )
				{
					backColor=SystemColors.Control ;
					foreColor=SystemColors.ControlDarkDark ;
				}
				else
				{
					backColor=this.BackColor ;
					foreColor=ControlPaint.DarkDark(this.BackColor);
				}
			}
		}

		protected override void OnPaintBackground(PaintEventArgs pevent)
		{
			try
			{
				Color backColor,foreColor;
				this.GetColors( out backColor, out foreColor);

				pevent.Graphics.Clear(backColor);
			
				//draw the border 
				if(this.borderStyle==BorderStyle.Fixed3D )
					ControlPaint.DrawBorder3D(pevent.Graphics,pevent.ClipRectangle,Border3DStyle.Sunken );
				else
					if(this.borderStyle==BorderStyle.FixedSingle)
					ControlPaint.DrawBorder3D(pevent.Graphics,pevent.ClipRectangle,Border3DStyle.Flat );


				//this.edit.BackColor=Color.Red;
				if(!this.rectImage.IsEmpty && this.imageIndex>-1 && this.imageIndex<this.imgList.Images.Count )
				{
					// draw the image 
					Rectangle sourceRect = new Rectangle(new Point(0,0),Util.GetMinSize(this.imgList.ImageSize,this.rectImage.Size));
					if(this.Enabled  )
						pevent.Graphics.DrawImage(this.imgList.Images[this.imageIndex],this.rectImage,sourceRect,GraphicsUnit.Pixel );
					else
						ControlPaint.DrawImageDisabled(pevent.Graphics,this.imgList.Images[this.imageIndex],this.rectImage.X,this.rectImage.Y,backColor);

				}

				if(!this.rectEllipsis.IsEmpty && this.Enabled )
				{
					this.DrawBackgroundEllipsis(pevent,backColor);
					this.DrawEllipsis(new Point(this.rectEllipsis.X+this.rectEllipsis.Width/2,this.rectEllipsis.Y+this.rectEllipsis.Height/2 ),pevent.Graphics );
				}
			}
			catch(Exception ex)
			{
			throw new Exception("ExtendedDataGrid.FormatTextBox.OnPaintBackground Exception : "+ex.Message );
			}
		}


		

		private void DrawBackgroundGradCentered(System.Windows.Forms.PaintEventArgs pevent,Color backColor)
		{

			try
			{
				Rectangle r=this.rectEllipsis  ;

				if (r.Height>12)
					r=new Rectangle(r.Left,r.Y+r.Height/2-6,r.Width,12);
				//pevent.Graphics.FillRectangle(Brushes.Green,r);
				PointF[] points=this.GetPoints(r);


				PathGradientBrush pgb=new PathGradientBrush(points);
				if(this.mouseIsDown)
					pgb.CenterColor=backColor ;
				else
					pgb.CenterColor=System.Drawing.SystemColors.Control ;

				pgb.SurroundColors=new Color[]{backColor};
				pgb.CenterPoint=new Point(r.X+r.Width/2,r.Y+r.Height/2 );
				pevent.Graphics.FillPolygon(pgb,points);
			}
			catch
			{}
		
		}


		private void DrawEllipsis(PointF center, Graphics g)
		{
			Color c1,c2;
			Brush br;
			try
			{
				if(this.mouseIsEntered )
				{
					c1=System.Drawing.SystemColors.ControlDarkDark ;
					c2=Color.White ;
				}
				else
				{
					c1=Color.White;
					c2=System.Drawing.SystemColors.ControlDarkDark ;
				}

				if (mouseIsDown)
				{
					br=new SolidBrush(SystemColors.ControlDark );
					g.FillRectangle(br, new RectangleF(center.X - 5, center.Y - 1, 3, 3));
					g.FillRectangle(br, new RectangleF(center.X - 1, center.Y - 1, 3, 3));
					g.FillRectangle(br, new RectangleF(center.X + 3, center.Y - 1, 3, 3));
					br.Dispose();
					br=Brushes.White ;
					g.FillRectangle(br,new RectangleF(center.X - 4, center.Y , 1, 1));
					g.FillRectangle(br,new RectangleF(center.X , center.Y , 1, 1));
					g.FillRectangle(br,new RectangleF(center.X +4, center.Y , 1, 1));
				}
				else
				{
					br=new SolidBrush(c1);
					g.FillRectangle(br, new RectangleF(center.X  - 4, center.Y, 2, 2));
					g.FillRectangle(br, new RectangleF(center.X, center.Y, 2, 2));
					g.FillRectangle(br, new RectangleF(center.X + 4, center.Y, 2, 2));
					br.Dispose();
					br=new SolidBrush(Color.FromArgb(200, c2));
					g.FillRectangle(br, new RectangleF(center.X - 5, center.Y - 1, 2, 2));
					g.FillRectangle(br, new RectangleF(center.X - 1, center.Y - 1, 2, 2));
					g.FillRectangle(br, new RectangleF(center.X + 3, center.Y - 1, 2, 2));
					br.Dispose();
				}
			}
			catch(Exception ex)
			{
				throw new Exception("FormatTextBox.DrawEllipsis Error : "+ex.Message );
			}
		}


		private PointF[]  GetPoints(Rectangle r) 
		{
			int nDim  = 25;
			PointF[] result=new PointF[nDim+1];
			float power = 0.3F;
			int i;
			float x,y;
			try
			{
				for(i=1;i<=nDim;i++)
				{
					x = (float)Math.Sin(2 * Math.PI * i / nDim);
					if( x > 0)
						x = 1 +  x + (1 - x) * power;
					else
						x =(1 + x)  * (1-power);
					x*=r.Width/2;
	
					y=(float)Math.Cos(2 * Math.PI * i / nDim);
					if (y > 0 )
						y = 1 +  y + (1 - y)  * power;
					else
						y = (1 + y)  *(1- power);
					y*=r.Height / 2;
		
					result[i - 1] = new PointF(x+r.X, y+r.Y );
				}
				result[nDim] = result[0];
			}
			catch(Exception ex)
			{
				throw new Exception("FormatTextBox.GetPoints Error : "+ex.Message );
			}

			return result;
		}


		private void DrawBackgroundEllipsis(PaintEventArgs pevent,Color backColor)
		{
			if(this.rectEllipsis.Height<3)
				return ;
			if (this.ellipsisBackStyle== EllipsisBackgroundStyle.GradientCentered )
				this.DrawBackgroundGradCentered(pevent,backColor);
			else
			{
				LinearGradientBrush br1;
				LinearGradientBrush br2;

				Color c1 ,c2;
				c1=this.BackColor ; c2=(this.mouseIsDown?this.BackColor: SystemColors.Control) ;
				Rectangle r1,r2;
				switch(this.ellipsisBackStyle )
				{
					case EllipsisBackgroundStyle.GradientHorizontal :
					{
						r1=this.rectEllipsis ;
						r1.Width=r1.Width /2;
						br1=new LinearGradientBrush(r1,c1,c2 ,LinearGradientMode.Horizontal );
						r2=r1; r2.Offset(r1.Width,0 );
						br2=new LinearGradientBrush(r2,c2,c1 ,LinearGradientMode.Horizontal );
						break;
					}
					case EllipsisBackgroundStyle.GradientVertical :
					{
						r1=this.rectEllipsis ;
						r1.Height=r1.Height/2+4;
						br1=new LinearGradientBrush(r1,c1,c2 ,LinearGradientMode.Vertical  );
						r2=r1; r2.Y+=pevent.ClipRectangle.Height /2;
						br2=new LinearGradientBrush(r2,c2,c1 ,LinearGradientMode.Vertical );
						break;
					}
					case EllipsisBackgroundStyle.ButtonStyle  :
					{
						r1=this.rectEllipsis ;
						r2=r1;
						br1=new LinearGradientBrush(r1,c1,c2,LinearGradientMode.Horizontal);
						br2=new LinearGradientBrush(r2,c1,c2,LinearGradientMode.Horizontal);
						break;
					}
					default   :
					{
						r1=this.rectEllipsis ;
						r2=r1;
						c1=backColor;c2=backColor;
						br1=new LinearGradientBrush(r1,c1,c2,LinearGradientMode.Horizontal);
						br2=new LinearGradientBrush(r2,c1,c2,LinearGradientMode.Horizontal);
						break;
					}
				}
				pevent.Graphics.FillRectangle(br1,r1);
				pevent.Graphics.FillRectangle(br2,r2);
				if (this.ellipsisBackStyle==EllipsisBackgroundStyle.ButtonStyle )
				{
					ButtonState buttonState;
					if(this.mouseIsDown || this.borderStyle==BorderStyle.FixedSingle  || this.borderStyle==BorderStyle.None )
						buttonState=ButtonState.Flat;
					else
						buttonState=ButtonState.Normal ;

					if(!this.Enabled)
						buttonState|=ButtonState.Inactive ;

					ControlPaint.DrawButton(pevent.Graphics ,this.rectEllipsis ,buttonState );
				}
			}
		}


		protected override void WndProc(ref Message m)
		{
			if(m.Msg==0x101) //WM_KEYUP
				return ;
			base.WndProc (ref m);
		}


		#endregion

		#region <<  PUBLIC >>

		public void SelectAll()
		{
		this.edit.SelectAll();
		}

		public string SelectedText
		{
			get{return this.edit.SelectedText;}
			set{if(value!=null && value.Length>0)
				this.edit.SelectedText=value;}
		}

		


		public void Select(int start,int length)
		{
			this.edit.Select(start,length);
		}
		#endregion 

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.edit = new DataGridTextBox();
			this.SuspendLayout();
			// 
			// edit
			// 
			this.edit.BorderStyle = System.Windows.Forms.BorderStyle.None;
			this.edit.Location = new System.Drawing.Point(40, 8);
			this.edit.Name = "edit";
			this.edit.TabIndex = 0;
			this.edit.Text = "";
			// 
			// FormatTextBox
			// 
			this.Controls.Add(this.edit);
			this.Name = "FormatTextBox";
			this.Size = new System.Drawing.Size(248, 32);
			this.Load += new System.EventHandler(this.FormatTextBox_Load);
			this.ResumeLayout(false);

		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region <<  PROTECTED >>

		protected virtual void OnEllipsisClick(EventArgs e)
		{
			if(this.ellipsis_Click!=null)	
			{
					this.ellipsis_Click(this,e);
			}
		}

		#endregion

		#region <<   PRIVATE  METHODS >>

		private void FormatTextBox_Load(object sender, System.EventArgs e)
		{
			this.RedimControls();
		}

		private void RedimControls()
		{
			Rectangle clientRect=this.ClientRectangle ;
			// rectImage
			int imageWidth = 0 ;
			if (this.imgList!=null && this.imageIndex>-1 && this.imageIndex<this.imgList.Images.Count && this.imgList.ImageSize.Width +2*sp <clientRect.Width )
			{
				this.rectImage=new Rectangle(clientRect.X   ,
											clientRect.Y,
											this.imgList.ImageSize.Width ,
											Math.Min(clientRect.Height ,this.imgList.ImageSize.Height ));
				if (this.rectImage.Height<clientRect.Height)
					this.rectImage=Util.CenterRectVertically(clientRect,this.rectImage);
				imageWidth=this.rectImage.Width+sp;
			}
			else
				this.rectImage=Rectangle.Empty ;

			//TetxBox bounds 
			int textboxWidth=clientRect.Right-imageWidth-2*DataGridFormatTextBox.sp-(this.showEllipsis?DataGridFormatTextBox.ellipsisWidth:0);

			if(textboxWidth>0 )
			{
				if (this.edit.Multiline)
					this.edit.Bounds=new Rectangle(imageWidth+DataGridFormatTextBox.sp,
						clientRect.Y,
						textboxWidth+sp,
						clientRect.Height);
				else
					this.edit.Bounds=new Rectangle(imageWidth+DataGridFormatTextBox.sp,
						clientRect.Y,
						textboxWidth+sp,
						this.edit.Height);
				if(this.edit.Height<clientRect.Height )
					this.edit.Bounds=Util.CenterRectVertically(clientRect,this.edit.Bounds );

			}
			else
			{
				this.edit.Bounds=Rectangle.Empty ;
				textboxWidth=0;
			}

			//rectEllipsis
			if(this.showEllipsis )
			{
				int elpsWidth=clientRect.Width-imageWidth-textboxWidth-sp;
				if(elpsWidth>=DataGridFormatTextBox.ellipsisWidth)
					elpsWidth=DataGridFormatTextBox.ellipsisWidth;
				else
					elpsWidth=0;

				if (elpsWidth>0)
					this.rectEllipsis=new Rectangle(clientRect.Right-elpsWidth,clientRect.Y,elpsWidth,clientRect.Height );
				else
					this.rectEllipsis=Rectangle.Empty ;
			}
			else
				this.rectEllipsis=Rectangle.Empty ;

			this.Invalidate();

		}

		private void edit_MultilineChanged(object sender, EventArgs e)
		{
			this.RedimControls();
		}

		private void edit_FontChanged(object sender, EventArgs e)
		{
			this.RedimControls();
		}

		private void edit_BorderStyleChanged(object sender, EventArgs e)
		{
			this.edit.BorderStyle=BorderStyle.None ;
		}

		private void FormatTextBox_EnabledChanged(object sender, EventArgs e)
		{
			this.edit.Enabled=this.Enabled ;
		}


		#endregion


		private void edit_TextChanged(object sender, EventArgs e)
		{
			this.OnTextChanged(new EventArgs());
		}
	}

	public enum EllipsisBackgroundStyle
	{
		GradientHorizontal,
		GradientVertical, 
		GradientCentered,
		ButtonStyle,
		None
	}


}
