using System;
using System.ComponentModel ;
using System.Runtime.InteropServices;


namespace ExtendedDataGrid.ColumnStyles
{
	/// <summary>
	/// This is the classic DataGridTextBoxColumn 
	/// </summary>
	[ComVisibleAttribute(false)]
	public class DataGridTextBoxColumn : System.Windows.Forms.DataGridTextBoxColumn, ExtendedDataGrid.ICommonColumnStyle
	{
		private ResizeType resizeType=ResizeType.Proportional ;
		private int colIndex=-1;
		private double widthPercent=0.0f;

		#region << Constructors   >>

		public DataGridTextBoxColumn():this(null){}

		public DataGridTextBoxColumn(PropertyDescriptor prop):base(prop){}

		#endregion 


		[Description("Specify how the column will resize when the grid will resize"),
		DefaultValue(ResizeType.Fixed ),Category("Behaviour")]
		ResizeType ICommonColumnStyle.ResizeType 
		{
			get{return this.resizeType ;}
			set{this.resizeType=value;}
		}
		
		double ICommonColumnStyle.WidthPercent 
		{
			get{return this.widthPercent;}
			set{this.widthPercent=value;}
		}

		[ReadOnly(true),Browsable(false),Description("The index of this column in the current table style ")]
		public int ColumnIndex
		{
			get
			{
				if(this.colIndex==-1)
					if(this.DataGridTableStyle!=null && this.DataGridTableStyle.GridColumnStyles.Contains(this))
						this.colIndex=this.DataGridTableStyle.GridColumnStyles.IndexOf(this);

				return this.colIndex;
			}
		}

		protected  override void ReleaseHostedControl()
		{
			this.colIndex=-1;
			base.ReleaseHostedControl ();
		}
		
		public void Refresh()
		{
		this.Invalidate();
		}

		protected override void SetDataGridInColumn(System.Windows.Forms.DataGrid value)
		{
			base.SetDataGridInColumn (value);
		}
	}
}
