using System;
using System.Reflection ;
using System.Drawing;
using System.Windows.Forms ;
using System.ComponentModel ;
using System.Data ;
using ExtendedDataGrid.ColumnStyles ;
using ExtendedDataGrid.Controls.Events;




// description 



namespace ExtendedDataGrid
{

	/// <summary>
	/// Class XTComboBox 
	/// This class is used in DataGridComboBoxColumn.
	/// The new properties : 
	/// <list type="bullet">
	/// <item>
	/// <description>ImageList</description>
	/// </item>
	/// <item>
	/// <description>DefaultImageIndex</description>
	/// </item>
	/// <item>
	/// <description>DefaultIndent</description>
	/// </item>
	/// </list>
	/// give you the possibility to associate to images to the items 
	/// <seealso cref="XTComboBox.SetItemStyle"/>
	/// </summary>
	/// 
	[ToolboxBitmap(typeof(XTComboBox))]
	public class XTComboBox : System.Windows.Forms.ComboBox
	{
		#region << Fields >>

		private ImageList imageList=null;
		private event ComboSetItemStyleEventHandler setStyle=null;

		private int defaultImageIndex=-1;
		private string imageIndexMember="";

		private string indentMember="";
		private int defaultIndent=1;
		internal  bool  DoNotFireSelectedIndexChanged=false;

		private Color selectedItemBackColor = System.Drawing.SystemColors.ActiveCaption ;
		private Color selectedItemForeColor = System.Drawing.SystemColors.ActiveCaptionText ;

		private int yMargin=1;

		#endregion 

		#region << CONSTRUCTORS  >>

		public XTComboBox()
		{
		base.DrawMode=System.Windows.Forms.DrawMode.OwnerDrawVariable   ;
		base.DropDownStyle=ComboBoxStyle.DropDownList ;
		base.ItemHeight=this.Font.Height+this.yMargin*2; 
		this.DataBindings.CollectionChanged+=new CollectionChangeEventHandler(DataBindings_CollectionChanged);
		}

		#endregion 

		#region << Properties >>

		public Color SelectedItemBackColor
		{
			get
			{
				return this.selectedItemBackColor;
			}
			set
			{
			this.selectedItemBackColor=value;
			}
		}


		protected bool ShouldSerializeSelectedItemBackColor()
		{
		return this.selectedItemBackColor!=System.Drawing.SystemColors.ActiveCaption ;
		}

		public void ResetSelectedItemBackColor()
		{
		this.selectedItemBackColor=System.Drawing.SystemColors.ActiveCaption ;
		}


		public Color SelectedItemForeColor
		{
			get
			{
				return this.selectedItemForeColor;
			}
			set
			{
				this.selectedItemForeColor=value;
			}
		}
		protected bool ShouldSerializeSelectedItemForeColor()
		{
			return this.selectedItemBackColor!=System.Drawing.SystemColors.ActiveCaptionText ;
		}

		public void ResetSelectedItemForeColor()
		{
			this.selectedItemBackColor=System.Drawing.SystemColors.ActiveCaptionText ;
		}

		[Category("Data"),
		DefaultValueAttribute(""),
		RefreshProperties(RefreshProperties.Repaint),
		Editor("System.Windows.Forms.Design.DataMemberFieldEditor,System.Design", typeof(System.Drawing.Design.UITypeEditor)),
		Description("Optional. You can specify the name of a field containing indexes for the images of the items")]
		public string ImageIndexMember
		{
			get{return this.imageIndexMember;}
			set{
				this.imageIndexMember=value;
				if(value.IndexOf(".")>-1)
					this.imageIndexMember=value.Substring(value.IndexOf(".")+1);
			}
		}


		[Category("Data"),
		DefaultValueAttribute(""),
		RefreshProperties(RefreshProperties.Repaint),
		Editor("System.Windows.Forms.Design.DataMemberFieldEditor,System.Design", typeof(System.Drawing.Design.UITypeEditor)),
		Description("Optional. You can specify the name of a field containing indents of the items ")]
		public string IndentMember
		{
			get{return this.indentMember;}
			set{
				this.indentMember=value;
				if(value.IndexOf(".")>-1)
					this.indentMember=value.Substring(value.IndexOf(".")+1);
			}
		}

		
		[DefaultValue(null),RefreshProperties(RefreshProperties.Repaint),
		Category("Layout"),Browsable(true)]
		public ImageList ImageList 
		{
				get{return this.imageList;}   
			set
			{
				if (value==null)
					this.defaultImageIndex =-1;
				this.imageList=value;
				if(value!=null)
					if(this.ItemHeight<Math.Max(this.ItemHeight,value.ImageSize.Height ))
						this.ItemHeight=Math.Max(this.ItemHeight,value.ImageSize.Height );
			}
		}

		public bool ShouldSerializeImageList()
		{
			return this.imageList!=null;
		}

		public void ResetImageList()
		{
			this.imageList=null;
		}


		/// <summary>
		/// The default image index 
		/// </summary>
		[Description("The default image index  "),
		Browsable(true),DefaultValue(-1),
		Category("Layout"),
		TypeConverter(typeof(ImageIndexConverter)),System.ComponentModel.EditorAttribute("System.Windows.Forms.Design.ImageIndexEditor",typeof(System.Drawing.Design.UITypeEditor))]
		public int DefaultImageIndex
		{
			get{return this.defaultImageIndex;}   
			set
			{	
				this.defaultImageIndex=value;
				this.Invalidate();
			}
		}


		[DefaultValue(1),Browsable(true),Category("Layout")]
		public int DefaultIndent
		{
			get{return this.defaultIndent;}
			set
			{
				if(value>=0)
					this.defaultIndent=value;
			}
		}


		public event ComboSetItemStyleEventHandler  SetItemStyle
		{
			add
			{
				if (value!=null)
					this.setStyle+=value;
			}
			remove{
				try
				{
					this.setStyle-=value;
				}
				catch(Exception ex )
				{
					throw new Exception(ex.Message );
				}
			}


		}


		[ReadOnly(true),DefaultValue(System.Windows.Forms.DrawMode.OwnerDrawVariable ),Browsable(true)]
		public new System.Windows.Forms.DrawMode DrawMode
		{
			get
			{
			return base.DrawMode;
			}
//			set
//			{
//			base.DrawMode=value;
//			}
		}


//		[DefaultValue(System.Windows.Forms.ComboBoxStyle.DropDownList )]
//		public new System.Windows.Forms.ComboBoxStyle DropDownStyle
//		{
//			get{return base.DropDownStyle ;}
//			set{base.DropDownStyle=value;}
//		}

		public override Font Font
		{
			get
			{
				return base.Font;
			}
			set
			{
				base.Font = value;
				if (value!=null)
					this.ItemHeight=Math.Max(this.ItemHeight,base.Font.Height);
			}
		}


		#endregion

		#region <<  OVERRIDES   >>

		protected virtual void OnSetItemStyle(ComboSetItemStyleEventArgs e)
		{
			if(this.setStyle!=null)
			{
				this.setStyle(this,e);
			}
		}

		protected override void WndProc(ref Message m)
		{
			if(m.Msg==0x101) //WM_KEYUP
				return ;
			base.WndProc (ref m);
		}

		protected override void OnDataSourceChanged(EventArgs e)
		{
			base.OnDataSourceChanged (e);
			if(this.DataSource==null)
			{
			this.ValueMember="";
			this.DisplayMember="";
			this.imageIndexMember="";
			this.indentMember="";
			}
		}

		protected override void OnDisplayMemberChanged(EventArgs e)
		{
			base.OnDisplayMemberChanged (e);
			if(base.DisplayMember.IndexOf(".")>-1)
				base.DisplayMember=base.DisplayMember.Substring(base.DisplayMember.IndexOf(".")+1);
		}

		protected override void OnValueMemberChanged(EventArgs e)
		{
			base.OnValueMemberChanged (e);
			if(base.ValueMember.IndexOf(".")>-1)
				base.ValueMember=base.ValueMember.Substring(base.ValueMember.IndexOf(".")+1);
		}


		protected override void OnMeasureItem(MeasureItemEventArgs e)
		{
			if(this.imageList!=null)
				e.ItemHeight=4+Math.Max(this.imageList.ImageSize.Height,this.Font.Height );
			else
				e.ItemHeight=4+this.Font.Height ;
			
		}

		protected override void OnSelectedIndexChanged(EventArgs e)
		{
			if(this.DoNotFireSelectedIndexChanged==false)
				base.OnSelectedIndexChanged (e);
		}

		protected override void OnDrawItem(DrawItemEventArgs e)
		{
				ValIdxIndentText dati;
				if(e.Index>-1)
					dati=this.GetValIdxIndentOfItem(this.Items[e.Index]);
				else
				{
					dati.ImageIndex=this.defaultImageIndex ;
					dati.Indent=this.defaultIndent ;
					dati.Text="";
					dati.Val=DBNull.Value ;
				}
				
				ComboSetItemStyleEventArgs es;
				if((e.State & DrawItemState.Selected) == DrawItemState.Selected )
					es=new ComboSetItemStyleEventArgs(this.selectedItemBackColor,this.selectedItemForeColor,dati.ImageIndex ,dati.Indent,e.Index,e.State );
				else
				 es=new ComboSetItemStyleEventArgs(e.BackColor,e.ForeColor,dati.ImageIndex ,dati.Indent,e.Index,e.State );
				
			OnSetItemStyle(es);

				int indent;
				Rectangle bounds=e.Bounds ;
				if((e.State & DrawItemState.ComboBoxEdit)==DrawItemState.ComboBoxEdit )
				{
					indent=0;
					bounds.Height=this.ItemHeight ;
				}
				else
					indent=es.CurrentItemIndent ;
				Brush br=new SolidBrush(es.CurrentBackColor );

				e.Graphics.FillRectangle(br,bounds);


				Rectangle imageRect=Rectangle.Empty ;Rectangle textRect=Rectangle.Empty ;
				int offset=0;
				int imageWidth=indent;

				if(this.imageList!=null )
				{
					offset=2;
					imageRect=new Rectangle(bounds.X+indent,bounds.Y,this.imageList.ImageSize.Width ,this.imageList.ImageSize.Height);
					imageWidth+= this.imageList.ImageSize.Width+offset;
					imageRect=CenterRectVertically(bounds,imageRect);
				}

				if(imageWidth<bounds.Width)
				{
					textRect=new Rectangle(bounds.X+imageWidth,
						bounds.Y ,bounds.Width-imageWidth,
						base.Font.Height );
					textRect=CenterRectVertically(bounds ,textRect );
				}
			
				// draw the text
				if(!textRect.IsEmpty)
				{
					Brush brush;
					if(this.Enabled)
						brush=new SolidBrush(es.CurrentForeColor );
					else
						brush=new SolidBrush(ControlPaint.Dark(es.CurrentBackColor ));

					e.Graphics.DrawString(dati.Text,base.Font,brush,textRect,null);
					brush.Dispose();
				}
				if (imageRect.IsEmpty)
					es.CurrentImageIndex=0;

				//draw the image
				if(!imageRect.IsEmpty)
				{
					if(es.CurrentImageIndex<this.imageList.Images.Count && es.CurrentImageIndex>-1  )
					
						if(this.Enabled )
							if(this.imageList.ImageSize.Height>imageRect.Height )
							{
								Rectangle sourceRect=new Rectangle(0,0,imageRect.Width,imageRect.Height );
								e.Graphics.DrawImage(this.imageList.Images[es.CurrentImageIndex],imageRect,sourceRect,GraphicsUnit.Pixel );
							}
							else
								e.Graphics.DrawImageUnscaled(this.imageList.Images[es.CurrentImageIndex],imageRect);
						else
							ControlPaint.DrawImageDisabled(e.Graphics ,this.imageList.Images[es.CurrentImageIndex],imageRect.X,imageRect.Y,base.BackColor);
//					else
//						if(es.CurrentImageIndex>-1)
//						throw new ArgumentOutOfRangeException("CurrentImageIndex",es.CurrentImageIndex,"XTComboBox.OnDrawItem Error :\nThe specified image index does not exist in the list");
				}
				if((e.State & DrawItemState.Selected) == DrawItemState.Selected )
					e.DrawFocusRectangle();
		}


		#endregion 

		#region << PROTECTED >>

		/// <summary>
		/// return the index of the image, the indentation and the Text for the corrisponding item of a ceratin value 
		/// </summary>
		/// <param name="keyVal"></param>
		/// <returns></returns>
		internal ValIdxIndentText GetValIdxIndentOfValue(object keyVal)
		{
			ValIdxIndentText result=new ValIdxIndentText(this.defaultImageIndex ,null,this.defaultIndent ,"");
			ValIdxIndentText tmp=new ValIdxIndentText(-1,null,0,"");
			foreach(object itm in this.Items )
			{
				tmp=this.GetValIdxIndentOfItem(itm,BindingType.SelectedValue);
				if(keyVal.Equals(tmp.Val))
				{
					result=tmp;
					break;
				}
			}
			return result;
		}


		internal  ValIdxIndentText GetValIdxIndentOfValue(object keyVal,BindingType bindTo)
		{
			ValIdxIndentText result=new ValIdxIndentText(this.defaultImageIndex ,null,this.defaultIndent ,"");
			ValIdxIndentText tmp=new ValIdxIndentText(-1,null,0,"");
			object foundItm=null;
			foreach(object itm in this.Items )
			{
				tmp=this.GetValIdxIndentOfItem(itm,bindTo);
				if(keyVal.Equals(tmp.Val))
				{
					result=tmp;
					foundItm=itm;
					break;
				}
			}
			if(foundItm!=null)
			{
				ComboSetItemStyleEventArgs e=new ComboSetItemStyleEventArgs(this.BackColor,this.ForeColor,result.ImageIndex,result.Indent,this.Items.IndexOf(foundItm),DrawItemState.Default );
				OnSetItemStyle(e);
				result.ImageIndex=e.CurrentImageIndex ;
				result.Indent=e.CurrentItemIndent ;
			}
			return result;
		}

		protected override void OnResize(EventArgs e)
		{
			base.OnResize (e);
			if(this.Width<20)
				this.Width=20;
		}


		internal ValIdxIndentText GetValIdxIndentOfItem(object itm,BindingType bindTo )
		{
			ValIdxIndentText result=new ValIdxIndentText(this.defaultImageIndex ,null,this.defaultIndent ,this.GetItemText(itm));
			DataRowView drv=itm as DataRowView;
			if(drv==null)
			{
				if(itm is ValueType && itm.GetType().IsPrimitive )
					if(bindTo==BindingType.SelectedIndex )
						result.Val =this.Items.IndexOf(itm);
					else
						result.Val=itm;
				else
				{
					if(itm is string)
						if(bindTo==BindingType.SelectedIndex )
							result.Val =this.Items.IndexOf(itm);
						else
							result.Val=itm;
					else
					{
						if(this.ValueMember.Length!=0)
							result.Val=GetPropVal(itm,this.ValueMember);
						if(this.imageIndexMember.Length!=0)
							result.ImageIndex=System.Convert.ToInt32( GetPropVal(itm,this.imageIndexMember ));
						else
							result.ImageIndex=this.defaultImageIndex ;
						if(this.indentMember.Length!=0)
							result.Indent=System.Convert.ToInt32(GetPropVal(itm,this.indentMember ));
						else
							result.Indent=this.defaultIndent ;
					}
				}
			}
			else
			{
				try
				{
					if(this.ValueMember.Length!=0)
						result.Val=drv[this.ValueMember];
					if(this.imageIndexMember.Length==0 || drv[this.imageIndexMember].Equals(DBNull.Value ))
						result.ImageIndex=this.defaultImageIndex ;
					else
						result.ImageIndex=System.Convert.ToInt32(drv[this.imageIndexMember ]);

					if(this.indentMember.Length==0 || drv[this.indentMember].Equals(DBNull.Value ))
						result.Indent=this.defaultIndent ;
					else
						result.Indent=System.Convert.ToInt32(drv[this.indentMember]);
				}
				catch(ArgumentOutOfRangeException  ex)
				{
					throw ex;
				}
			}
			return result;
		}


		internal ValIdxIndentText GetValIdxIndentOfItem(object itm)
		{
			ValIdxIndentText result=new ValIdxIndentText(this.defaultImageIndex ,null,this.defaultIndent ,"");
			result.Text=this.GetItemText(itm);
			DataRowView drv=itm as DataRowView;
			if(drv==null)
			{
				if(itm is ValueType && itm.GetType().IsPrimitive )
					result.Val =itm;
				else
					if(itm is string)
						result.Val=itm;
					else
					{
						if(this.ValueMember.Length!=0)
							result.Val=GetPropVal(itm,this.ValueMember);
						if(this.imageIndexMember.Length!=0)
							result.ImageIndex=System.Convert.ToInt32(GetPropVal(itm,this.imageIndexMember ));
						else
							result.ImageIndex=this.defaultImageIndex ;
						if(this.indentMember.Length!=0)
							result.Indent=System.Convert.ToInt32(GetPropVal(itm,this.indentMember ));
						else
							result.Indent=this.defaultIndent ;
					}
			}
			else
			{
				try
				{
					if(this.ValueMember.Length!=0)
						result.Val=drv[this.ValueMember];

					if(this.imageIndexMember.Length==0 || drv[this.imageIndexMember].Equals(DBNull.Value ))
						result.ImageIndex=this.defaultImageIndex ;
					else
						result.ImageIndex=System.Convert.ToInt32(drv[this.imageIndexMember ]);

					if(this.indentMember.Length==0 || drv[this.indentMember].Equals(DBNull.Value ))
						result.Indent=this.defaultIndent ;
					else
						result.Indent=System.Convert.ToInt32(drv[this.indentMember]);
				}
				catch(ArgumentException  ex)
				{
					throw ex;
				}
			}
			if(result.ImageIndex.Equals(DBNull.Value))
				result.ImageIndex=this.defaultImageIndex ;
			return result;
		}


		protected virtual object GetPropVal(object itm , string member)
		{
			object result=null;
			System.Reflection.PropertyInfo propInfo;
			propInfo=itm.GetType().GetProperty(member);
			if(propInfo==null || !propInfo.CanRead )
			{
				if(member==this.ValueMember || member.Length!=0)
					throw new Exception("ComboboxX DataSource exception : \nLa DataSource non contiene una proprieta col nome "+this.DisplayMember );
			}
			else
			{
				result=propInfo.GetValue(itm,null);
			}
			return result;
		}

		protected virtual object GetValueOfItem(object itm)
		{
			return GetValueOfItem(itm,BindingType.SelectedValue );
		}
		protected virtual object GetValueOfItem(object itm,BindingType bindTo)
		{
			/// si deve eliminare 
			object result=null;
				DataRowView drv=itm as DataRowView;
				if(drv==null)
				{
					if(itm is ValueType && itm.GetType().IsPrimitive )
						result=itm;
					else
						if(itm is string)
							result=itm;
						else
						{
							switch(bindTo)
							{
								case BindingType.SelectedValue :
									if(this.ValueMember.Length!=0)
										result=this.GetPropVal(itm,this.ValueMember);
										break;
								case BindingType.SelectedIndex  :
									result=this.SelectedIndex ;
									break;
								case BindingType.Text: 
									result=this.Text ;
									break;
							}
						}
				}
				else
				{
					try
					{
						switch(bindTo)
						{
							case BindingType.SelectedValue :
								if(this.ValueMember.Length!=0)
									result=drv[this.ValueMember];
								break;
							case BindingType.SelectedIndex  :
								result=this.SelectedIndex ;
								break;
							case BindingType.Text: 
								result=this.Text ;
								break;
						}
					}
					catch(ArgumentOutOfRangeException  ex)
					{
						throw ex;
					}
				}

			return result;
		}

		

		#endregion

		#region << PRIVATES >>

		private Rectangle CenterRectVertically(Rectangle container,Rectangle child)
		{
		Rectangle result=child;
			if(container.Height>child.Height)
			{
				result=new Rectangle(child.X,container.Y+(container.Height-child.Height)/2,child.Width,child.Height);
			}
		if (result.Bottom>container.Bottom )
			result.Intersect(container);
		return result ;
		}


		#endregion

		private void DataBindings_CollectionChanged(object sender, CollectionChangeEventArgs e)
		{
			if(e.Action==CollectionChangeAction.Add)
			{
				Binding bnd=(Binding)e.Element ;
				if(bnd.IsBinding==false)
				{
					DataView dv=bnd.DataSource as DataView; 
					dv.EndInit();
				}
			}
		}


	}

	internal struct ValIdxIndentText
	{
		public int ImageIndex;
		public object Val;
		public int Indent;
		public string Text;
		public ValIdxIndentText(int imgIndex, object val, int indent,string text)
		{
			this.ImageIndex=imgIndex;this.Val=val;
			this.Indent=indent;this.Text=text;
		}
	}

	namespace Controls
	{
		namespace Events
		{
			#region <<  SetItemStyle : delegate & eventargs >>

			public delegate void ComboSetItemStyleEventHandler(object sender, ComboSetItemStyleEventArgs e);
			public class ComboSetItemStyleEventArgs
			{
				//Fields 
				private Color currentBackColor;
				private Color currentForeColor;
				private int currentImageIndex;
				private int currentIndent;
				private int currentItemIndex;
				private System.Windows.Forms.DrawItemState state;

				//Constructor  
				public ComboSetItemStyleEventArgs(Color backColor,Color foreColor,int imageIndex,int indent,int itemIndex,DrawItemState stato)
				{
					this.currentIndent=indent;
					this.currentImageIndex=imageIndex;
					this.currentItemIndex=itemIndex;
					this.currentBackColor=backColor;
					this.currentForeColor=foreColor;
					this.state=stato;
		
				}

				//Properties 
				/// <summary>
				/// Gets/Sets the BackColor of the item to be paint
				/// </summary>
				public Color CurrentBackColor
				{
					get{return this.currentBackColor ;}
					set{this.currentBackColor=value;}
				}
		
				/// <summary>
				/// Gets/Sets the ForeColor for the text of the item being painted 
				/// </summary>
				[Description(" Gets/Sets the ForeColor for the text of the item being painted ")]
				public Color CurrentForeColor
				{
					get{return this.currentForeColor ;}
					set{this.currentForeColor=value;}
				}
		
				/// <summary>
				/// Gets/Sets the index of the image of the item being painted 
				/// </summary>
				[Description("Gets/Sets the index of the image of the item being painted ")]
				public int CurrentImageIndex
				{
					get{return this.currentImageIndex ;}
					set{this.currentImageIndex=value;}
				}

				/// <summary>
				/// Gets/Sets the indentation of the item being painted 
				/// </summary>
				[Description("Gets/Sets the indentation of the item being painted")]
				public int CurrentItemIndent
				{
					get{return this.currentIndent ;}
					set{this.currentIndent=value;}
				}

				/// <summary>
				/// ReadOnly. 
				/// The index of the item being painted, that is the item for which this event has been raised
				/// </summary>
				[Description("The index of the item being painted, that is the item for which this event has been raised")]
				public int CurrentItemIndex
				{get{return this.currentItemIndex ;}}

				/// <summary>
				/// The state of the item being painted 
				/// </summary>
				[Description("The state of the item being painted")]
				public DrawItemState State
				{
					get{return this.state ;}
				}
			}


			#endregion
		} // end namespace Events
	}
	

}
